# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from scipy.stats import norm
from math import pi
from statsmodels.stats.power import tt_ind_solve_power, zt_ind_solve_power
from mozanalysis.bq import BigQueryContext
from mozanalysis.experiment import TimeSeriesResult
from statsmodels.stats.proportion import samplesize_proportions_2indep_onetail
import numpy as np
import pandas as pd
from mozanalysis.metrics import Metric
from typing import List, Union, Dict


def sample_size_curves(
    df: pd.DataFrame,
    metrics_list: list,
    solver,
    effect_size: Union[float, Union[np.ndarray, pd.Series, List[float]]] = 0.01,
    power: Union[float, Union[np.ndarray, pd.Series, List[float]]] = 0.80,
    alpha: Union[float, Union[np.ndarray, pd.Series, List[float]]] = 0.05,
    **solver_kwargs,
) -> Dict[str, pd.DataFrame]:
    """
    Loop over a list of different parameters to produce sample size estimates given
    those parameters. A single parameter in [effect_size, power, alpha] should
    be passed a list; the sample size curve will be calculated with this as
    the variable.

    Args:
        df: A pandas DataFrame of queried historical data.
        metrics_list (list of mozanalysis.metrics.Metric): List of metrics
            used to construct the results df from HistoricalTarget. The names
            of these metrics are used to return results for sample size
            calculation for each.
        solver (any function that returns sample size as function of
            effect_size, power, alpha): The solver being used to calculate sample
            size.
        effect_size (float or ArrayLike, default .01): For test of differences in
            proportions, the absolute difference; for tests of differences in mean,
            the percent change.
        alpha (float or ArrayLike, default .05): Significance level for the experiment.
        power (float or ArrayLike, default .90): Probability of detecting an effect,
            when a significant effect exists.
        **solver_kwargs (dict): Arguments necessary for the provided solver.

    Returns:
        A dictionary of pd.DataFrame objects. An item in the dictionary is
        created for each metric in metric_list, containing a DataFrame of sample
        size, population size, and population proportion at each value of the iterable
        parameter.
    """

    params = {"effect_size": effect_size, "power": power, "alpha": alpha}
    sim_var = [k for k, v in params.items() if type(v) in [list, np.ndarray, pd.Series]]

    if len(sim_var) != 1:
        raise ValueError(
            "Exactly one of effect_size, power, and alpha must be ArrayLike"
        )

    sim_var = sim_var[0]
    test_vals = params[sim_var]
    del params[sim_var]

    results = {}
    for metric in metrics_list:
        sample_sizes = []
        for v in test_vals:
            sample_sizes.append(
                {
                    **{sim_var: v},
                    **solver(df, [metric], **{sim_var: v}, **params, **solver_kwargs)[
                        metric.name
                    ],
                }
            )
        results[metric.name] = pd.DataFrame(sample_sizes)

    return results


def difference_of_proportions_sample_size_calc(
    df: pd.DataFrame,
    metrics_list: List[Metric],
    effect_size: float = 0.01,
    alpha: float = 0.05,
    power: float = 0.90,
    outlier_percentile: float = 99.5,
) -> dict:

    """
    Perform sample size calculation for an experiment to test for a
    difference in proportions.

    Args:
        df: A pandas DataFrame of queried historical data.
        metrics_list (list of mozanalysis.metrics.Metric): List of metrics
            used to construct the results df from HistoricalTarget. The names
            of these metrics are used to return results for sample size
            calculation for each
        effect_size (float, default .01): Difference in proportion for the
            minimum detectable effect --
            effect_size = p(event under alt) - p(event under null)
        alpha (float, default .05): Significance level for the experiment.
        power (float, default .90): Probability of detecting an effect,
            when a significant effect exists.
        outlier_percentile(float, default .995): Percentile at which to trim
            each columns.

    Returns:
        A dictionary. Keys in the dictionary are the metrics column names from
        the DataFrame; values are the required sample size to achieve the
        desired power for that metric.
    """

    def _get_sample_size_col(col):

        p = np.percentile(df[col], q=[outlier_percentile])[0]
        mean = df.loc[df[col] <= p, col].mean()
        p2 = mean + effect_size

        return samplesize_proportions_2indep_onetail(
            diff=effect_size, prop2=p2, power=power, ratio=1, alpha=alpha, value=0
        )

    metric_names = [m.name for m in metrics_list]
    results = {}
    for col in metric_names:
        sample_size = _get_sample_size_col(col)
        pop_percent = 100.0 * (sample_size / len(df))
        results[col] = {
            "sample_size": sample_size,
            "population_size": len(df),
            "population_percent": pop_percent,
        }
    return results


def z_or_t_ind_sample_size_calc(
    df: pd.DataFrame,
    metrics_list: List[Metric],
    test: str = "z",
    effect_size: float = 0.01,
    alpha: float = 0.05,
    power: float = 0.90,
    outlier_percentile: float = 99.5,
) -> dict:

    """
    Perform sample size calculation for an experiment based on independent
    samples t or z tests.

    Args:
        df: A pandas DataFrame of queried historical data.
        metrics_list (list of mozanalysis.metrics.Metric): List of metrics
            used to construct the results df from HistoricalTarget. The names
            of these metrics are used to return results for sample size
            calculation for each
        test (str, default `z`): `z` or `t` to indicate which solver to use
        effect_size (float, default .01): Percent change in metrics
            expected as a result of the experiment treatment
        alpha (float, default .05): Significance level for the experiment.
        power (float, default .90): Probability of detecting an effect,
            when a significant effect exists.
        outlier_percentile(float, default .995): Percentile at which to trim
            each columns.

    Returns:
        A dictionary. Keys in the dictionary are the metrics column names from
        the DataFrame; values are the required sample size to achieve the
        desired power for that metric.
    """
    tests = {
        "normal": zt_ind_solve_power,
        "z": zt_ind_solve_power,
        "t": tt_ind_solve_power,
    }
    solver = tests[test]

    def _get_sample_size_col(col):

        p = np.percentile(df[col], q=[outlier_percentile])[0]
        sd = df.loc[df[col] <= p, col].std()
        mean = df.loc[df[col] <= p, col].mean()
        es = (effect_size * mean) / sd

        return solver(effect_size=es, alpha=alpha, power=power, nobs1=None)

    metric_names = [m.name for m in metrics_list]
    results = {}
    for col in metric_names:
        sample_size = _get_sample_size_col(col)
        pop_percent = 100.0 * (sample_size / len(df))
        results[col] = {
            "sample_size": sample_size,
            "population_size": len(df),
            "population_percent": pop_percent,
        }
    return results


def empirical_effect_size_sample_size_calc(
    res: TimeSeriesResult,
    bq_context: BigQueryContext,
    metric_list: list,
    quantile: float = 0.90,
    power: float = 0.80,
    alpha: float = 0.05,
    parent_distribution: str = "normal",
) -> dict:

    """
    Perform sample size calculation with empirical effect size and
    asymptotic approximation of Wilcoxen-Mann-Whitney U Test. Empirical effect size
    is estimated using a quantile of week-to-week changes over
    the course of the study, and the variance in the test statistic is
    estimated as a quantile of weekly variance in metrics. Sample
    size calculation is based on the asymptotic relative efficiency (ARE) of
    the U test to the T test (see Stapleton 2008, pg 266, or
    https://www.psychologie.hhu.de/fileadmin/redaktion/Fakultaeten/
    Mathematisch-Naturwissenschaftliche_Fakultaet/Psychologie/AAP/gpower/GPowerManual.pdf)

    Args:
        res: A TimeSeriesResult, generated by
            mozanalysis.sizing.HistoricalTarget.get_time_series_data.
        bq_context: A mozanalysis.bq.BigQueryContext object that handles downloading
            time series data from BigQuery.
        metrics_list (list of mozanalysis.metrics.Metric): List of metrics
            used to construct the results df from HistoricalTarget. The names
            of these metrics are used to return results for sample size
            calculation for each.
        quantile (float, default .90): Quantile used to calculate the effect size
            as the quantile of week-to-week metric changes and the variance of
            the mean.
        alpha (float, default .05): Significance level for the experiment.
        power (float, default .90): Probability of detecting an effect,
            when a significant effect exists.
        parent_distribution (str, default "normal"): Distribution of the parent data;
            must be normal, uniform, logistic, or laplace.

    Returns:
        A dictionary. Keys in the dictionary are the metrics column names from
        the DataFrame; values are the required sample size to achieve the
        desired power for that metric.
    """

    def _mann_whitney_solve_sample_size_approximation(
        effect_size, std, alpha=0.05, power=0.8, parent_distribution="normal"
    ):

        """
        Wilcoxen-Mann-Whitney rank sum test sample size calculation,
        based on asymptotic efficiency relative to the t-test.
        """
        rel_effect_size = effect_size / std
        are = {
            "uniform": 1.0,
            "normal": pi / 3.0,
            "logistic": 9.0 / (pi**2),
            "laplace": 2.0 / 3.0,
        }

        if parent_distribution not in are.keys():
            raise ValueError(f"Parent distribution must be in {are.keys()}")

        t_sample_size = tt_ind_solve_power(
            effect_size=rel_effect_size, power=power, alpha=alpha
        )

        return t_sample_size * are[parent_distribution]

    res_mean, pop_size = res.get_aggregated_data(
        bq_context=bq_context, metric_list=metric_list, aggregate_function="AVG"
    )
    res_mean.sort_values(by="analysis_window_start", ascending=True, inplace=True)

    res_std, _ = res.get_aggregated_data(
        bq_context=bq_context, metric_list=metric_list, aggregate_function="STDDEV"
    )

    effect_size = {}
    std = {}

    for m in metric_list:
        res_mean["diff"] = res_mean[m.name].diff().abs()
        m_quantile = res_mean["diff"].quantile(q=quantile, interpolation="nearest")
        m_std = res_std[m.name].quantile(q=quantile, interpolation="nearest")

        effect_size[m.name] = {
            "value": m_quantile,
            "period_start_day": res_mean.loc[
                res_mean["diff"] == m_quantile, "analysis_window_start"
            ].values[0],
        }
        std[m.name] = {
            "value": m_std,
            "period_start_day": res_std.loc[
                res_std[m.name] == m_std, "analysis_window_start"
            ].values[0],
        }

    size_dict = {
        m.name: {
            "effect_size": effect_size[m.name],
            "std_dev": std[m.name],
            "sample_size": _mann_whitney_solve_sample_size_approximation(
                effect_size=effect_size[m.name]["value"],
                std=std[m.name]["value"],
                power=power,
                alpha=alpha,
                parent_distribution=parent_distribution,
            ),
        }
        for m in metric_list
    }

    for k in size_dict.keys():
        size_dict[k]["population_size"] = pop_size
        size_dict[k]["population_percent"] = 100.0 * (
            size_dict[k]["sample_size"] / pop_size
        )

    return size_dict


def poisson_diff_solve_sample_size(
    df: pd.DataFrame,
    metrics_list: List[Metric],
    effect_size: float = 0.01,
    alpha: float = 0.05,
    power: float = 0.90,
    outlier_percentile: float = 99.5,
) -> dict:
    """
    Sample size for test of difference of Poisson rates,
    based on Poisson rate's asymptotic normality.

    Args:
        df: A pandas DataFrame of queried historical data.
        metrics_list (list of mozanalysis.metrics.Metric): List of metrics
            used to construct the results df from HistoricalTarget. The names
            of these metrics are used to return results for sample size
            calculation for each
        test (str, default `z`): `z` or `t` to indicate which solver to use
        effect_size (float, default .01): Percent change in metrics
            expected as a result of the experiment treatment
        alpha (float, default .05): Significance level for the experiment.
        power (float, default .90): Probability of detecting an effect,
            when a significant effect exists.
        outlier_percentile(float, default .995): Percentile at which to trim
            each columns.

    Returns:
        A dictionary. Keys in the dictionary are the metrics column names from
        the DataFrame; values are the required sample size to achieve the
        desired power for that metric.
    """

    def _get_sample_size_col(col):

        p = np.percentile(df[col], q=[outlier_percentile])[0]
        sd = df.loc[df[col] <= p, col].std()
        mean = df.loc[df[col] <= p, col].mean()
        es = (effect_size * mean) / sd

        z_alpha = norm.ppf(1 - alpha / 2)
        z_power = norm.ppf(power)

        denom = (es / (z_alpha + z_power)) ** 2
        sample_size = (mean + es) / denom
        return sample_size

    metric_names = [m.name for m in metrics_list]
    results = {}
    for col in metric_names:
        sample_size = _get_sample_size_col(col)
        pop_percent = 100.0 * (sample_size / len(df))
        results[col] = {
            "sample_size": sample_size,
            "population_size": len(df),
            "population_percent": pop_percent,
        }
    return results



from cheap_lint import sql_lint

import mozanalysis.metrics.desktop as mad
import mozanalysis.segments.desktop as msd
from mozanalysis.experiment import TimeLimits
from mozanalysis.sizing import HistoricalTarget
from mozanalysis.segments import Segment, SegmentDataSource
from mozanalysis.metrics import Metric, DataSource


def test_multiple_datasource():
    test_target = HistoricalTarget("test_targ", "2022-01-01", 7, 3)

    tl = TimeLimits.for_single_analysis_window(
        first_enrollment_date="2022-01-01",
        last_date_full_data="2022-01-04",
        analysis_start_days=0,
        analysis_length_dates=3,
    )

    test_sds = SegmentDataSource("test_ds", "test_table")
    test_seg = Segment("test_seg", test_sds, "TEST AGG SELECT STATEMENT", "", "")

    target_sql = test_target.build_targets_query(
        time_limits=tl, target_list=[msd.new_unique_profiles, test_seg]
    )

    sql_lint(target_sql)

    assert "ds_1" in target_sql


def test_query_not_detectably_malformed():

    test_target = HistoricalTarget("test_targ", "2022-01-01", 7, 3)

    tl = TimeLimits.for_single_analysis_window(
        first_enrollment_date="2022-01-01",
        last_date_full_data="2022-01-04",
        analysis_start_days=0,
        analysis_length_dates=3,
    )

    target_sql = test_target.build_targets_query(
        time_limits=tl, target_list=[msd.new_unique_profiles]
    )

    sql_lint(target_sql)

    metrics_sql = test_target.build_metrics_query(
        time_limits=tl, metric_list=[], targets_table="targets"
    )

    sql_lint(metrics_sql)


def test_megaquery_not_detectably_malformed():

    test_target = HistoricalTarget("test_targ", "2022-01-01", 7, 3)

    tl = TimeLimits.for_single_analysis_window(
        first_enrollment_date="2022-01-01",
        last_date_full_data="2022-01-04",
        analysis_start_days=0,
        analysis_length_dates=3,
    )

    target_sql = test_target.build_targets_query(
        time_limits=tl,
        target_list=[s for s in msd.__dict__.values() if isinstance(s, Segment)],
    )

    sql_lint(target_sql)

    metrics_sql = test_target.build_metrics_query(
        time_limits=tl,
        metric_list=[
            m
            for m in mad.__dict__.values()
            if isinstance(m, Metric) and "experiment_slug" not in m.select_expr
        ],
        targets_table="targets",
    )

    sql_lint(metrics_sql)


def test_custom_query_override_target():
    test_target = HistoricalTarget("test_targ", "2022-01-01", 7, 3)

    test_sds = SegmentDataSource(
        "test_ds",
        "test_table",
    )

    test_seg = Segment("test_seg", test_sds, "TEST AGG SELECT STATEMENT", "", "")

    tl = TimeLimits.for_single_analysis_window(
        first_enrollment_date="2022-01-01",
        last_date_full_data="2022-01-04",
        analysis_start_days=0,
        analysis_length_dates=3,
    )

    custom_query = """
    SELECT * from custom_query_target_table
    """

    target_sql = test_target.build_targets_query(
        time_limits=tl, target_list=[test_seg], custom_targets_query=custom_query
    )

    assert (
        "custom_query_target_table" in target_sql
        and "TEST AGG SELECT STATEMENT" not in target_sql
    )


def test_resolve_missing_column_names():
    test_ds = DataSource(
        name="test_ds",
        from_expr="SELECT client_id FROM test_set",
        client_id_column=None,
        submission_date_column=None,
    )

    test_metric = Metric(name="test_metric", data_source=test_ds, select_expr="")

    tl = TimeLimits.for_single_analysis_window(
        first_enrollment_date="2022-01-01",
        last_date_full_data="2022-01-13",
        analysis_start_days=0,
        analysis_length_dates=7,
    )

    test_sds = SegmentDataSource(
        name="test_sds",
        from_expr="SELECT * FROM test_sds",
        client_id_column=None,
        submission_date_column=None,
    )

    test_seg = Segment(name="test_seg", data_source=test_sds, select_expr="")

    test_targ = HistoricalTarget(
        experiment_name="test_exp",
        start_date="2022-01-01",
        analysis_length=7,
    )

    targets_sql = test_targ.build_targets_query(time_limits=tl, target_list=[test_seg])

    metrics_sql = test_targ.build_metrics_query(
        time_limits=tl, metric_list=[test_metric], targets_table="test_targ_enrollment"
    )

    assert "None" not in targets_sql
    assert "None" not in metrics_sql



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from typing import Optional

from metric_config_parser.config import ConfigCollection


class _ConfigLoader:
    """
    Loads config files from an external repository.

    Config objects are converted into mozanalysis native types.
    """

    config_collection: Optional[ConfigCollection] = None

    @property
    def configs(self) -> ConfigCollection:
        configs = getattr(self, "_configs", None)
        if configs:
            return configs

        if self.config_collection is None:
            self.config_collection = ConfigCollection.from_github_repo()
        self._configs = self.config_collection
        return self._configs

    def get_metric(self, metric_slug: str, app_name: str):
        from mozanalysis.metrics import Metric

        metric_definition = self.configs.get_metric_definition(metric_slug, app_name)
        if metric_definition is None:
            raise Exception(f"Could not find definition for metric {metric_slug}")

        return Metric(
            name=metric_definition.name,
            select_expr=self.configs.get_env()
            .from_string(metric_definition.select_expression)
            .render(),
            friendly_name=metric_definition.friendly_name,
            description=metric_definition.friendly_name,
            data_source=self.get_data_source(
                metric_definition.data_source.name, app_name
            ),
            bigger_is_better=metric_definition.bigger_is_better,
        )

    def get_data_source(self, data_source_slug: str, app_name: str):
        from mozanalysis.metrics import DataSource

        data_source_definition = self.configs.get_data_source_definition(
            data_source_slug, app_name
        )
        if data_source_definition is None:
            raise Exception(
                f"Could not find definition for data source {data_source_slug}"
            )

        return DataSource(
            name=data_source_definition.name,
            from_expr=data_source_definition.from_expression,
            client_id_column=data_source_definition.client_id_column,
            submission_date_column=data_source_definition.submission_date_column,
            experiments_column_type=None
            if data_source_definition.experiments_column_type == "none"
            else data_source_definition.experiments_column_type,
            default_dataset=data_source_definition.default_dataset,
        )

    def get_segment(self, segment_slug: str, app_name: str):
        from mozanalysis.segments import Segment

        segment_definition = self.configs.get_segment_definition(segment_slug, app_name)
        if segment_definition is None:
            raise Exception(f"Could not find definition for segment {segment_slug}")

        return Segment(
            name=segment_definition.name,
            data_source=self.get_segment_data_source(
                segment_definition.data_source.name, app_name
            ),
            select_expr=self.configs.get_env()
            .from_string(segment_definition.select_expression)
            .render(),
            friendly_name=segment_definition.friendly_name,
            description=segment_definition.description,
        )

    def get_segment_data_source(self, data_source_slug: str, app_name: str):
        from mozanalysis.segments import SegmentDataSource

        data_source_definition = self.configs.get_segment_data_source_definition(
            data_source_slug, app_name
        )
        if data_source_definition is None:
            raise Exception(
                f"Could not find definition for segment data source {data_source_slug}"
            )

        return SegmentDataSource(
            name=data_source_definition.name,
            from_expr=data_source_definition.from_expression,
            window_start=data_source_definition.window_start,
            window_end=data_source_definition.window_end,
            client_id_column=data_source_definition.client_id_column,
            submission_date_column=data_source_definition.submission_date_column,
            default_dataset=data_source_definition.default_dataset,
        )


ConfigLoader = _ConfigLoader()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

# These metric definitions are deprecated.
# Instead use the metric slug to reference metrics defined
# in https://github.com/mozilla/metric-hub

from mozanalysis.config import ConfigLoader

clients_daily = ConfigLoader.get_data_source("clients_daily", "firefox_desktop")

search_clients_engines_sources_daily = ConfigLoader.get_data_source(
    "search_clients_engines_sources_daily", "firefox_desktop"
)

search_clients_daily = search_clients_engines_sources_daily

main_summary = ConfigLoader.get_data_source("main_summary", "firefox_desktop")

events = ConfigLoader.get_data_source("events", "firefox_desktop")

normandy_events = ConfigLoader.get_data_source("normandy_events", "firefox_desktop")

main = ConfigLoader.get_data_source("main", "firefox_desktop")

crash = ConfigLoader.get_data_source("crash", "firefox_desktop")

cfr = ConfigLoader.get_data_source("cfr", "firefox_desktop")

activity_stream_events = ConfigLoader.get_data_source(
    "activity_stream_events", "firefox_desktop"
)


active_hours = ConfigLoader.get_metric("active_hours", "firefox_desktop")

uri_count = ConfigLoader.get_metric("uri_count", "firefox_desktop")

search_count = ConfigLoader.get_metric("search_count", "firefox_desktop")

tagged_search_count = ConfigLoader.get_metric("tagged_search_count", "firefox_desktop")

tagged_follow_on_search_count = ConfigLoader.get_metric(
    "tagged_follow_on_search_count", "firefox_desktop"
)

ad_clicks = ConfigLoader.get_metric("ad_clicks", "firefox_desktop")

searches_with_ads = ConfigLoader.get_metric("searches_with_ads", "firefox_desktop")

organic_search_count = ConfigLoader.get_metric(
    "organic_search_count", "firefox_desktop"
)

unenroll = ConfigLoader.get_metric("unenroll", "firefox_desktop")

view_about_logins = ConfigLoader.get_metric("view_about_logins", "firefox_desktop")

view_about_protections = ConfigLoader.get_metric(
    "view_about_protections", "firefox_desktop"
)

connect_fxa = ConfigLoader.get_metric("connect_fxa", "firefox_desktop")

pocket_rec_clicks = ConfigLoader.get_metric("pocket_rec_clicks", "firefox_desktop")

pocket_spoc_clicks = ConfigLoader.get_metric("pocket_spoc_clicks", "firefox_desktop")

days_of_use = ConfigLoader.get_metric("days_of_use", "firefox_desktop")

qualified_cumulative_days_of_use = ConfigLoader.get_metric(
    "qualified_cumulative_days_of_use", "firefox_desktop"
)

disable_pocket_clicks = ConfigLoader.get_metric(
    "disable_pocket_clicks", "firefox_desktop"
)

disable_pocket_spocs_clicks = ConfigLoader.get_metric(
    "disable_pocket_spocs_clicks", "firefox_desktop"
)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

# These metric definitions are deprecated.
# Instead use the metric slug to reference metrics defined
# in https://github.com/mozilla/metric-hub

from mozanalysis.config import ConfigLoader

baseline = ConfigLoader.get_data_source("baseline", "firefox_ios")

events = ConfigLoader.get_data_source("events", "firefox_ios")

metrics = ConfigLoader.get_data_source("metrics", "firefox_ios")

baseline_ping_count = ConfigLoader.get_metric("baseline_ping_count", "firefox_ios")

metric_ping_count = ConfigLoader.get_metric("metric_ping_count", "firefox_ios")

first_run_date = ConfigLoader.get_metric("first_run_date", "firefox_ios")



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

# These segment definitions are deprecated.
# Instead use the segment slug to reference segments defined
# in https://github.com/mozilla/metric-hub

from mozanalysis.config import ConfigLoader

clients_last_seen = ConfigLoader.get_segment_data_source(
    "clients_last_seen", "firefox_desktop"
)


regular_users_v3 = ConfigLoader.get_segment("regular_users_v3", "firefox_desktop")

new_or_resurrected_v3 = ConfigLoader.get_segment(
    "new_or_resurrected_v3", "firefox_desktop"
)

weekday_regular_v1 = ConfigLoader.get_segment("weekday_regular_v1", "firefox_desktop")

allweek_regular_v1 = ConfigLoader.get_segment("allweek_regular_v1", "firefox_desktop")

new_unique_profiles = ConfigLoader.get_segment("new_unique_profiles", "firefox_desktop")



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
import numpy as np
import pandas as pd

from mozanalysis.frequentist_stats.sample_size import z_or_t_ind_sample_size_calc
from mozanalysis.metrics.desktop import search_clients_daily, uri_count


def test_sample_size_calc_desktop():
    df = pd.DataFrame(
        {
            search_clients_daily.name: np.random.normal(size=100),
            uri_count.name: np.random.normal(size=100),
        }
    )

    res = z_or_t_ind_sample_size_calc(df, [search_clients_daily, uri_count])

    assert all([c in res.keys() for c in df.columns])

    assert res[search_clients_daily.name]["sample_size"] > 1000000
    assert res[uri_count.name]["sample_size"] > 1000000
